#!/bin/bash
debug=yes_

function format () {

echo Formatting partition $1 with type $2

if test "$2" = "rfs"; then 
 echo y | fsdisk.sh reiser none none $1 &> /dev/null
elif test "$2" = "rfsnotail"; then
 echo y | fsdisk.sh reiser none -o\ notail $1 &> /dev/null
elif test "$2" = "bmp"; then 
 echo y | fsdisk.sh reiser none none $1 &> /dev/null
elif test "$2" = "bmpnotail"; then
 echo y | fsdisk.sh reiser none -o\ notail $1 &> /dev/null
elif test "$2" = "xfs"; then
 echo y | fsdisk.sh xfs -f none $1 &> /dev/null
elif test "$2" = "jfs"; then
 echo y | fsdisk.sh jfs none none $1 &> /dev/null
elif test "$2" = "ext2"; then
 echo y | fsdisk.sh ext2 "-b 4096 -N 350000 -m 0" none $1 &> /dev/null
else
 echo unknown type $2
fi
}

function clsync() {
local dir=$1
if test -z "$dir"; then return; fi
local str=`mount | grep $dir`
local dev=`echo $str | cut -f 1 -d\ `
local type=`echo $str | cut -f 5 -d\ `
local opt=""

if test -n "`echo $str | grep notail`" ; then
 opt="-o notail"
fi

umount $dev
sleep 2
echo mounting $dev $dir -t $type $opt
mount $dev $dir -t $type $opt
}



function run () {
if test "$debug" = "yes"; then 
 echo $fdesc
 return 
fi

local wpart=1
local wdir=/mnt/scsi$wpart
local cpart=2
local cdir=/mnt/scsi$cpart
local workdir=`pwd`

local tmpdir=/tmp/results-root/apptest2/`date "+%s"`-$fdesc
mkdir -p $tmpdir

local i
local u
local sfile
local wsfile


if test -z "$loopstr"; then loopstr="rfs rfsnotail ext2"; fi
#for i in rfs rfsnotail ext2; do
for i in $loopstr; do

 format $wpart $i
 format $cpart $i
 sleep 10

if test -n "$preallocstr"; then
 /root/bin/preallocate -d $wdir $preallocstr
fi
  
 for u in 25 50 75 99; do
# for u in 10 15; do

#age
  sfile=$tmpdir/$fdesc-$i-$u
  echo working on $sfile
  time /root/bin/agesystem -d $wdir -u $u -l $lparam -m $mparam $tparam -z > $sfile
  echo ------------
  echo

#cp
  clsync $wdir
  format $cpart $i
  clsync $cdir
  wsfile=$tmpdir/$fdesc-$i-$u-copy
  cp $sfile $wsfile
  echo working on $wsfile
  cd $wdir
  time cp -R . $cdir
  echo ------------
  echo
  cd $workdir

  clsync $wdir
  clsync $cdir

#fibmap
  sfile=$tmpdir/$fdesc-$i-$u-fibmap
  echo working on $sfile
  time /root/bin/fibmap $wdir > $sfile
  echo ------------
  echo

  clsync $wdir
  clsync $cdir

#cp
  wsfile=$tmpdir/$fdesc-$i-$u-fibmap-copy
  echo working on $wsfile
  time /root/bin/fibmap $cdir > $wsfile
  echo ------------
  echo

  clsync $wdir
  clsync $cdir

#read
  sfile=$tmpdir/$fdesc-$i-$u-readtest
  echo working on $sfile
  time /root/bin/readtest -l $((1 * $u * 10000)) -d $wdir -z $rparam . > $sfile
  echo ------------
  echo

  clsync $wdir
  clsync $cdir

#cp
  wsfile=$tmpdir/$fdesc-$i-$u-readtest-copy
  echo working on $wsfile
  time /root/bin/readtest -l $((1 * $u * 10000)) -d $cdir -z $rparam . > $wsfile
  echo ------------
  echo

  clsync $wdir
  clsync $cdir

 done
done

sleep 10
}



#-------------------------------------------------
# done 
#-------------------------------------------------

loopstr="rfs rfsnotail ext2"

#distributionstandard=ds
rparam="-p 25000:250000"
tparam="-c 1:1"
preallocstr=""
mparam=4096:65536:1048576
lparam=0:4096:65536

fdesc=cmp-1GB-gauge-ds-1x1-smem-2410
#run ; 

#distributionstandard=ds
rparam="-p 25000:250000"
preallocstr=""
tparam="-c 100:10000"
mparam=4096:65536:1048576
lparam=0:4096:65536

fdesc=cmp-1GB-gauge-ds-100x100-smem-2410
#run ; 

#append
for n in 1000 5000 10000; do
for i in 4096; do 

fdesc=cmp-1GB-1x1-$n-pre-appfix-$i-smem-2410
preallocstr="-c 1:1 -n $n"
tparam="-c 1:1 -o 1089 -n $n -b $i -f"
rparam="-p 10:250000"
#run ; 

fdesc=cmp-1GB-1x1-$n-appfix-$i-smem-2410
preallocstr=""
tparam="-c 1:1 -o 1089 -n $n -b $i -f"
rparam="-p 10:250000"
#run ; 

done
done 

#append
for n in 1000 5000 10000 ; do
for i in 4096; do 

fdesc=cmp-1GB-1x1000-$n-pre-appfix-$i-smem-2410
preallocstr="-c 1:1000 -n $n"
tparam="-c 1:1000 -o 1089 -n $n -b $i -f"
rparam="-p 2500:250000"
#run ; 

fdesc=cmp-1GB-1x1000-$n-appfix-$i-smem-2410
preallocstr=""
tparam="-c 1:1000 -o 1089 -n $n -b $i -f"
rparam="-p 2500:250000"
#run ; 

done
done 


loopstr="rfs rfsnotail ext2"
#append
#for n in 1000 5000 10000 25000; do
for n in 5000 ; do
for i in 4096 12288 65536 ; do 

fdesc=cmp-1GB-1x1-$n-pre-appfix-$i-smem-248ac1
preallocstr="-c 1:1 -n $n"
tparam="-c 1:1 -o 1089 -n $n -b $i -f"
rparam="-p 10:250000"
#run ; 

done
done 

for n in 5000 ; do
for i in 4096 12288 65536 ; do 

fdesc=cmp-1GB-1x1-$n-appfix-$i-smem-248ac1
preallocstr=""
tparam="-c 1:1 -o 1089 -n $n -b $i -f"
rparam="-p 10:250000"
#run ; 

done
done 


#distributionstandard=ds
rparam="-p 10:250000"
tparam="-c 1:1"
mparam=4096:65536:1048576
lparam=0:4096:65536

fdesc=cmp-1GB-gauge-ds-1x1-smem-248ac1
#run ; 

#d2
mparam=16384:131072:2097152
lparam=4096:16384:131072

fdesc=cmp-1GB-gauge-d2-1x1-smem-248ac1
#run ; 

#d3
mparam=65536:524288:4194304
lparam=16384:65536:524288

fdesc=cmp-1GB-gauge-d3-1x1-smem-248ac1
#run ;

#distributionstandard=ds
rparam="-p 25000:250000"
tparam="-c 100:10000"
mparam=4096:65536:1048576
lparam=0:4096:65536

fdesc=cmp-1GB-gauge-ds-100x100-smem-248ac1
#run ; 

#d2
mparam=16384:131072:2097152
lparam=4096:16384:131072

fdesc=cmp-1GB-gauge-d2-100x100-smem-248ac1
#run ; 

#d3
mparam=65536:524288:4194304
lparam=16384:65536:524288

fdesc=cmp-1GB-gauge-d3-100x100-smem-248ac1
#run ;
