/*

Copyright 2001 
by Constantin Loizides <constantin.loizides@isg.de>


This file is part of the journaling-filesystem fragmentation project.
You find the files belonging to that project in the Makefile
coming together with this file.

Source code concerning that project is free software; 
you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  
USA

*/


#include "common.h"


#define PROGNAME "preallocate"
#define VERSION "0.3"


/* for common.h */
int verbose=0;
int blocksize=BUFFERSIZE;
char *dummybuffer;


/*for mount point info*/
int fdsave=-1; 
cl_String tmpdir="";

void usage(){
	printf("Usage of %s v%s\n",PROGNAME,VERSION);
	printf("\t -b blksize\t\t\tblocksize to use for buffered i/o\n");
	printf("\t -c dir_base:dir_counter\t\t\tstructure of subdirectories to create and use for aging\n");
	printf("\t -d dirname\t\t\tchange to directory \"dirname\"\n");
	printf("\t -f \t\t\t\tuse fixed length of bytes, given in blocksize\n");
	printf("\t -n number_of_files\t\tmaximum number of files to create\n");
	printf("\t -o access_mode\t\t\tset access mode to  \"access_mode\" \n\t    (eg. cw=%d, cws=%d, cwa=%d,cwas=%d, cwt=%d,cwts=%d)\n",O_CREAT|O_WRONLY,O_CREAT|O_WRONLY|O_SYNC,O_CREAT|O_WRONLY|O_APPEND,O_CREAT|O_WRONLY|O_APPEND|O_SYNC,O_CREAT|O_WRONLY|O_TRUNC,O_CREAT|O_WRONLY|O_TRUNC|O_SYNC);
	printf("\t -h,-?\t\t\t\tthis text\n");
	printf("\t -t\t\t\t\ttest input and exit\n");
	printf("\t -v\t\t\t\tbe verbose\n");
	exit(1);
}

inline int f2d(int fn,int dircounter)
{
	return (fn % dircounter);
}

inline int d2d(int dn,int dirbase)
{
	return (dn % dirbase);
}



int main(int argc,char **argv)
{
	int accessoptions=O_CREAT|O_WRONLY;
	int dircounter=5000;
	int dirbase=100;
	int testonly=0;
	int fixedlength=0;	
	long max_fileno=-1;
		
	/*parse options*/
	int temp;
	char ch;
	while((ch=getopt(argc,argv,"b:c:d:fl:n:m:o:u:h?s:vty:z"))!=EOF){
		switch(ch)	{	
			case 'b':
				temp=atoi(optarg);
				if(temp<1)
				{
					fprintf(stderr, "Can't set blocksize to \"%s\".\n", optarg);
					usage();
				} else blocksize=temp;
				break;
			case 'c':{
				int i=0;
				char *opttok=strtok(optarg,":");
				while((opttok!=NULL)&&(i++<2)){
					//printf("%d-%s\n",++i,opttok);
					temp=atoi(opttok);
					if(temp<0)
					{
						fprintf(stderr, "Can't set directory counter to \"%s\".\n", optarg);
						usage();
					} 
					else if(i==1) dirbase=temp;
					else dircounter=temp;
					opttok=strtok(NULL,":");
				}
				break;}
			case 'd':
				if(chdir(optarg))
				{
					fprintf(stderr, "Can't change to directory \"%s\".\n", optarg);
					usage();
				}
				else tmpdir=optarg;
				break;
			case 'f': fixedlength=1; break;
			case 'n':
				temp=atol(optarg);
				if(temp<0)
				{
					fprintf(stderr, "Can't set maximum filenumber to \"%s\".\n", optarg);
					usage();
				} else max_fileno=temp;
				break;
			case 'o':
				temp=atoi(optarg);
				if(temp<1)
				{
					fprintf(stderr, "Can't set access options to \"%s\".\n", optarg);
					usage();
				} else accessoptions=temp;
				break;
			case '?':
			case 'h': usage(); break;
			case 'v': verbose=1; break;
			case 't': testonly=1; break;
			default: usage(); break;
		}
	}

	dummybuffer=new char[blocksize];
	memset(dummybuffer,31,blocksize);

	if(testonly==1){
		cout << "accessoptions: " << accessoptions << endl;
		cout << "dircounter: " << dircounter << endl;
		cout << "max_filenumber: " << max_fileno << endl;
		cout << "dirbase: " << dirbase << endl;
		cout << "fixedlength: " << fixedlength << endl;
		cout << "blocksize: " << blocksize << endl;
		cout << "verbose: " << verbose << endl;
		exit(0);
	}

		
	fdsave=open(".",O_RDONLY);
	if(fdsave==-1){
		fprintf(stderr,"Error opening \"%s\", reason: %s\n",".",strerror(errno));
		exit(1);
	}		
	
	struct statfs buf_statfs;
	if(fstatfs(fdsave,&buf_statfs)==-1)
	{
		fprintf(stderr,"Error statfsing \"%s\", reason: %s\n",".",strerror(errno));
		exit(1);
	}


	
	unsigned long long filecounter=0;

	//create dirs
	cl_String dir(250);
	for(int i=0;i<dircounter;i++)
	{
		dir="verzeichnis-";
		dir+=d2d(i,dirbase);
		mkdir((char*)dir,S_IRUSR|S_IWUSR|S_IXUSR);
		dir+="/inner-";
		dir+=i;
		int ret=mkdir((char*)dir,S_IRUSR|S_IWUSR|S_IXUSR);
		if(ret==-1){
			dir="verzeichnis-";
			i=0;
			dir+=d2d(i,dirbase);
			dir+="/inner-";
			dir+=i;
			long no=countdirentries((const char*)dir);
			if(no<0) {
				fprintf(stderr,"Error creating dirs, exiting...\n");
				exit(1);
			} 
			else filecounter=(1+no)*dircounter;
			//printf("Found %llu\n",filecounter);
			break;
		}
	}

	//float fsu=(float)(100-100.*(float)buf_statfs.f_bfree/(float)buf_statfs.f_blocks);
	float fbls=buf_statfs.f_bfree;
	long  length=blocksize;

	if(fixedlength!=1){
		length=4096*(int)(fbls/max_fileno);
	}
	
	cl_String file(50);

	for(int filecounter=0;filecounter<max_fileno;filecounter++)
	{
			int dirnumber=f2d(filecounter,dircounter);
			dir="verzeichnis-";
			dir+=d2d(dirnumber,dirbase);
			dir+="/inner-";
			dir+=dirnumber;
			file=dir;
			file+="/datei-";
			file+=filecounter;
			writefile((char *)file,length,accessoptions);
			if(verbose) printf("Create file \"%s\" with length %ld\n",(char *)file,length);
	}

	sync();
	sync();
	
	for(int filecounter=0;filecounter<max_fileno;filecounter++)
	{
			int dirnumber=f2d(filecounter,dircounter);
			dir="verzeichnis-";
			dir+=d2d(dirnumber,dirbase);
			dir+="/inner-";
			dir+=dirnumber;
			file=dir;
			file+="/datei-";
			file+=filecounter;

			truncate((char*)file,4096);
			if(verbose) printf("Truncate file \"%s\"\n",(char *)file);
	}


	if(fdsave!=-1) close(fdsave);
	delete[] dummybuffer;
}
